package cn.com.ctop.toutiao.modules.material.service.impl;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.utils.*;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceCampaign;
import cn.com.ctop.toutiao.modules.material.mapper.ByteDanceCampaignMapper;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceCampaignService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 今日头条广告组信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */

@Service
@Slf4j
public class ByteDanceCampaignServiceImpl extends ServiceImpl<ByteDanceCampaignMapper, ByteDanceCampaign> implements IByteDanceCampaignService {
    /**
     * 每个广告主账号下最多可允许创建500个广告组，如超出需要先删除一部分广告组后才可继续创建。
     * @param token
     * @param campaignName
     * @param operation
     * @param budgetMode
     * @param budget
     * @param landingType
     * @param uniqueFk
     * @return
     */
    @Override
    public Map<String, Object> createCampaign(CtopOauthToken token, String campaignName, String operation, String budgetMode, String budget, String landingType, String uniqueFk) {
        Map<String,Object>result = new HashMap<>();
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_create");
        // 请求参数
        JSONObject param = new JSONObject();
        param.put("advertiser_id",token.getAccountId());
        param.put("campaign_name",campaignName);
        if(null!=operation&&!operation.trim().equals("")){
            param.put("operation",operation);
        }
        param.put("budget_mode", budgetMode);
        if(null != budget&&!budget.trim().equals("")){
            param.put("budget",budget);
        }
        param.put("landing_type", landingType);
        if(null != uniqueFk&&!uniqueFk.trim().equals("")){
            param.put("unique_fk",uniqueFk);
        }
        JSONObject data = HttpUtils.bytedancePostRequest(url,token.getAccessToken(),param);
        if(data == null){
            log.error("创建广告组异常=>param:{}",param.toJSONString());
            ResultMapUtils.setResultMap(result, StatusCode.BYTEDACNE_API_GROUP_CREATE_ERROR);
            return result;
        }
        Integer code = data.getInteger("code");
        String message = data.getString("message");
        if(null == code||!code.equals(0)){
            log.error("获取广告组数据异常=>{},parmas:{}",message,param.toJSONString());
            ResultMapUtils.setResultMap(result,-1,message,false);
            return result;
        }
        Long campaignId = data.getJSONObject("data").getLong("campaign_id");
        ResultMapUtils.setResultMap(result,StatusCode.COMMON_SUCCESS);
        result.put("campaign_id",campaignId);
        return result;
    }

    @Override
    public Map<String, Object> updateCampaign(CtopOauthToken token, String campaignId, String campaignName,String budgetMode, String budget) {
        Map<String, Object> resultMap = new HashMap<>();
        ByteDanceCampaign campaign = this.getById(campaignId);
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_update");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("campaign_id", campaignId);
        params.put("modify_time", campaign.getModifyTime());
        if (null != campaignName && !"".equals(campaignName.trim())) {
            params.put("campaign_name", campaignName);
        }
        if (null != budgetMode && !"".equals(budgetMode.trim())) {
            params.put("budget_mode", budgetMode);
        }
        if (null != budget) {
            params.put("budget", budget);
        }
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("广告组更新接口异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("广告组更新异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新异常");
            return resultMap;
        }
        if (null != campaignName && !"".equals(campaignName.trim())) {
            campaign.setName(campaignName);
        }
        if (null != budgetMode && !"".equals(budgetMode.trim())) {
            params.put("budget_mode", budgetMode);
            campaign.setBudgetMode(budgetMode);
        }
        if (null != budget) {
            campaign.setBudget(new BigDecimal(budget));
        }
        this.updateById(campaign);

        resultMap.put("code", 0);
        resultMap.put("message", "广告组信息修改成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> getAdvertiserCampaign(CtopOauthToken token, String ids, String date) {
        getAdvertiserCampaignByPageNumber(token, 1, ids, date);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("code", 0);
        resultMap.put("message", "获取广告組信息成功");
        return resultMap;
    }

    private void getAdvertiserCampaignByPageNumber(CtopOauthToken token, Integer pageNumber, String ids, String date) {
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_get");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        TreeMap<String, Object> params = new TreeMap<>();
        JSONObject filtering = new JSONObject();
        filtering.put("status","CAMPAIGN_STATUS_ALL");
        if (null != ids && !"".equals(ids.trim())) {
            String[] idsArray = ids.split(StringUtils.COMMA);
            JSONArray filterIdsArray = new JSONArray();
            for (int i = 0; i < idsArray.length; i++) {
                Long id = Long.parseLong(idsArray[i]);
                filterIdsArray.add(id);
            }
            filtering.put("ids", filterIdsArray);
        }
        if (null != date && !"".equals(date)) {
            filtering.put("campaign_create_time", date);
        }
        params.put("filtering", filtering.toJSONString());
        params.put("advertiser_id", token.getAccountId());
        params.put("page", pageNumber + "");
        String result = HttpUtils.httpGetRequest(url, headers, params);
        System.out.println(result);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");
        if (null == code || !code.equals(0)) {
            log.error("获取广告组信息接口异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            return;
        }
        JSONArray data = jsonObject.getJSONObject("data").getJSONArray("list");
        if (null == data || data.isEmpty()) {
            log.error("获取广告组信息不存在==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            return;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObject = data.getJSONObject(i);
            ByteDanceCampaign campaign = new ByteDanceCampaign(dataObject,token.getAccountId());
            this.saveOrUpdate(campaign);
        }
        getAdvertiserCampaignByPageNumber(token, pageNumber + 1, ids, date);
    }

    @Override
    public Map<String, Object> updateCampaignStatus(CtopOauthToken token, String campaignIds, String optStatus) {
        Map<String, Object> resultMap = new HashMap<>();
        JSONArray ids = new JSONArray();
        String[] getIds = campaignIds.split(StringUtils.COMMA);
        if (null != getIds && getIds.length > 0) {
            for (int i = 0; i < getIds.length; i++) {
                ids.add(Long.parseLong(getIds[i]));
            }
        }
        //2: 根据token以及用户id获取用户信息数据
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_update_status");
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("campaign_ids", ids);
        params.put("opt_status", optStatus);
        String result = HttpUtils.httpPostRequest(url, params, headers);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Integer code = jsonObject.getInteger("code");

        if (null == code || !code.equals(0)) {
            log.info("广告组更新状态接口异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新状态接口异常");
            return resultMap;
        }
        JSONObject data = jsonObject.getJSONObject("data");
        if (null == data) {
            log.info("广告组更新状态异常==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组更新状态异常");
            return resultMap;
        }
        JSONArray getCampaignIds = data.getJSONArray("campaign_ids");
        if (null != getCampaignIds && !getCampaignIds.isEmpty()) {
            for (int i = 0; i < getCampaignIds.size(); i++) {
                Long id = getCampaignIds.getLong(i);
                getAdvertiserCampaign(token,id+"",null);
            }
        }
        resultMap.put("code", 0);
        resultMap.put("message", "广告组状态修改成功");
        return resultMap;
    }
}
